/* $Id: cdrom.c,v 1.33 2005/07/08 18:37:57 matthew Exp $
 *
 * mcdp - command line CD playing
 * Copyright (C) 2003 Matthew Vernon <matthew@gnu.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Log: cdrom.c,v $
 * Revision 1.33  2005/07/08 18:37:57  matthew
 * handle SIGINT whilst playing
 *
 * Revision 1.32  2005/05/15 15:45:23  matthew
 * if len is 1, we have one track to play!
 *
 * Revision 1.31  2005/02/14 11:08:06  matthew
 * include string.h (for memset)
 *
 * Revision 1.30  2003/11/23 14:47:59  matthew
 * split freedb functions out into separate .c file
 *
 * Revision 1.29  2003/11/22 18:27:32  matthew
 * Add code to generate the CDDB query string
 *
 * Revision 1.28  2003/11/22 17:19:54  matthew
 * Separate out the discid-generating code from the code that prints it.
 *
 * Revision 1.27  2003/09/01 23:40:52  matthew
 * Add support for generating freedb &c. discids
 *
 * Revision 1.26  2003/04/19 20:11:57  matthew
 * remove debug code
 *
 * Revision 1.25  2003/04/19 19:07:57  matthew
 * looping and random play functions. Debugging still enabled
 *
 * Revision 1.24  2003/04/18 18:17:53  matthew
 * use termios to establish width of terminal
 *
 * Revision 1.23  2003/04/18 17:27:06  matthew
 * correct erroneous duplicate check for CDC_CLOSE_TRAY
 *
 * Revision 1.22  2003/04/18 17:25:08  matthew
 * Added capabilities query
 *
 * Revision 1.21  2003/04/16 09:22:35  matthew
 * make sure dir talks about the last track
 *
 * Revision 1.20  2003/04/06 16:50:43  matthew
 * Added previous and next track code
 *
 * Revision 1.19  2003/04/06 16:34:02  matthew
 * Add relative/absolute address functionality
 *
 * Revision 1.18  2003/04/05 09:43:28  matthew
 * remember to fill in tocentry stucture
 *
 * Revision 1.17  2003/04/05 09:33:09  matthew
 * directory function written, needs debugging
 *
 * Revision 1.16  2003/04/05 01:40:11  matthew
 * Added status query functionality
 *
 * Revision 1.15  2003/04/04 20:07:10  matthew
 * add get current track function
 *
 * Revision 1.14  2003/04/04 19:26:04  matthew
 * initialise track number variables
 *
 * Revision 1.13  2003/04/04 16:39:03  matthew
 * Added track specification parsing
 *
 * Revision 1.12  2003/04/04 16:25:35  matthew
 * update cd_play_tracks to cope with arguments being zero, obsoletes cd_play_disc
 *
 * Revision 1.11  2003/04/04 15:35:50  matthew
 * removed old debug code
 *
 * Revision 1.10  2003/04/04 15:19:16  matthew
 * Added pause/resume functionality
 *
 * Revision 1.9  2003/04/04 15:06:14  matthew
 * Added audio_status query function
 *
 * Revision 1.8  2003/04/04 14:44:05  matthew
 * Add copyright notice and CVS variables
 *
 *
 */

#include<sys/types.h>
#include<sys/time.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<sys/ioctl.h>
#include<linux/cdrom.h>
#include<termios.h>
#include<unistd.h>
#include<errno.h>
#include<time.h>
#include<signal.h>

#include "ferror.h"
#include "libmcdp.h"

int pipe_fds[2];

static struct cdrom_subchnl cd_get_subchnl(const int drive,
					   struct cdrom_subchnl *sc);
static unsigned long cd_raw_address(const int drive, const cdrom_addr_t type);
static unsigned long cd_raw_track_length(const int drive, 
					 unsigned char track);
static unsigned long cd_frames_track_length(const int drive, 
					    unsigned char track);
static int cd_seconds_disc_length(const int drive);
static cdrom_tracktype_t cd_type_track(const int drive, 
				       const unsigned char track);
static int display_capability(const char *string, int used, const int cols);
static int *list_all_tracks(const int drive, int **lt, int *len);
static int *trim_list(int **lt, int *len, int where);
static int *check_track_list(const int drive, int **lt, int *len);
static int *cycle_list(int **lt, int *len);
static int list_next_track(int **lt,int *len,const int loop,const int rndm);
static void wait_play_track(const int drive, const int t);
static void sigint_handler(int signum);


/*Deprecated - just use cd_play_tracks */
unsigned char cd_play_disc (int drive, const unsigned char start)
{
  unsigned char begin,finish;

  finish=cd_end_track(drive,CDROM_LAST_TRACK);
  if(start) begin=start;
  else begin=cd_end_track(drive,CDROM_FIRST_TRACK);

  return cd_play_tracks(drive,begin,finish);
}

unsigned char cd_play_tracks(const int drive, const unsigned char begin, const unsigned char finish)
{
  struct cdrom_ti ti;
  unsigned char a,z,last;

  if(begin)a=begin;
  else a=cd_end_track(drive,CDROM_FIRST_TRACK);
  last=cd_end_track(drive,CDROM_LAST_TRACK);
  if(finish)z=finish;
  else z=last;
  
  if(((finish)&&(finish>last))||((begin)&&(begin>last)))
    fatal_error("Specified track greater than number of tracks on CD!",0);

  memset (&ti,0,sizeof(struct cdrom_ti));
  ti.cdti_trk0=a;
  ti.cdti_trk1=z;
  /*Stop the cd first*/
  cd_stop_drive(drive);
  if(ioctl(drive,CDROMPLAYTRKIND,&ti))
    fatal_error("ioctl CDROMPLAYTRKIND",1);
  return begin;
}

/*always exits if failure*/
void cd_stop_drive(const int drive)
{
  if(ioctl(drive,CDROMSTOP))
    fatal_error("stopping cd",1);
}

void cd_load_drive(const int drive)
{
  if(ioctl(drive,CDROMCLOSETRAY))
    fatal_error("loading cd",1);
}

void cd_eject_drive(const int drive)
{
  if(ioctl(drive,CDROMEJECT))
    fatal_error("ejecting cd",1);
}

void cd_next_track(const int drive)
{
  unsigned char t=cd_current_track(drive);
  unsigned char last=cd_end_track(drive,CDROM_LAST_TRACK);
  if(t)t++;
  /*If not currently playing, t will be 0, so set it to 2*/
  else t=2;
  /*Check we're not trying to fall off the end...*/
  if(t>last)t=last;
  cd_play_tracks(drive,t,0);
}

void cd_prev_track(const int drive)
{
  unsigned char t=cd_current_track(drive);
  if(t>1)t--; /*if t=1, just go back to the beginning of that track*/
  if(t==0) fatal_error("Can't skip back when not playing!",0);
  cd_play_tracks(drive,t,0);
}

int cd_open_device(const char *path)
{
  int fd;

  if((fd=open(path,O_RDONLY|O_NONBLOCK))==-1){
    fatal_error("opening CD device",1);
  }
  return fd;
}

void cd_close_device(const int drive)
{
  if (-1==close(drive))
    fatal_error("closing CD drive",1);
}

void cd_pause_drive(const int drive)
{
  switch(cd_audio_status(drive)){
  case CDROM_AUDIO_PLAY:
    if(ioctl(drive,CDROMPAUSE)){
      fatal_error("pausing drive",1);
    }
    break;
  case CDROM_AUDIO_PAUSED:
    if(ioctl(drive,CDROMRESUME)){
      fatal_error("resuming drive",1);
    }
    break;
  case CDROM_AUDIO_COMPLETED:
  case CDROM_AUDIO_NO_STATUS:
    fprintf(stderr,
	    "Cannot pause drive: not currently playing nor paused\n");
    break;
  case CDROM_AUDIO_INVALID:
    fatal_error("Cannot pause drive - audio not supported",0);
  case CDROM_AUDIO_ERROR:
    fatal_error("Cannot pause drive - audio stopped due to error",0);
  default:
    fatal_error("Unknown status value returned",0);
  }
}  


unsigned char cd_end_track (const int drive,const cdrom_firstlast_t which)
{
  struct cdrom_tochdr tochdr;

  if(ioctl(drive,CDROMREADTOCHDR, &tochdr)){
    if(ENOMEDIUM==errno){
      fprintf(stderr,"disc not found, trying load\n");
      cd_load_drive(drive);
    }
    else fatal_error("ioctl CDROMREADTOCHDR",1);
  }

  switch(which)
    {
  case CDROM_FIRST_TRACK:
    return tochdr.cdth_trk0; break;
  case CDROM_LAST_TRACK:
    return tochdr.cdth_trk1; break;
  default:
    fatal_error("Reached non-sensical part of switch statement",0);
    break;
    }
}
  
int cd_audio_status(const int drive)
{
  struct cdrom_subchnl sc;
  sc=cd_get_subchnl(drive,&sc);
  return sc.cdsc_audiostatus; /*see cdrom.h for status codes*/
}

unsigned char cd_current_track(const int drive)
{
  struct cdrom_subchnl sc;
  sc=cd_get_subchnl(drive,&sc);
  if((sc.cdsc_audiostatus==CDROM_AUDIO_PLAY)
     ||(sc.cdsc_audiostatus==CDROM_AUDIO_PAUSED))
    return sc.cdsc_trk;
  else return 0;
}
  
static struct cdrom_subchnl cd_get_subchnl(const int drive,
					   struct cdrom_subchnl *sc)
{
  memset(sc,0,sizeof(struct cdrom_subchnl));
  /*I think CDROM_LBA would do here too, 
    but MSF is more widely supported?? 
  */
  sc->cdsc_format=CDROM_MSF;
  if(ioctl(drive,CDROMSUBCHNL, sc)){
    fatal_error("ioctl CDROMSUBCHNL",1);
  }
  return *sc;
}

/*absolute address (==time since beginning of CD) or
 * relative address (==time since beginning of track)
 * in frames
 */
static unsigned long cd_raw_address(const int drive, const cdrom_addr_t type)
{
  struct cdrom_subchnl sc;
  unsigned long frames=0;
  sc=cd_get_subchnl(drive,&sc);
  
  if(CDROM_ABSOLUTE==type){
    frames=sc.cdsc_absaddr.msf.frame;
    frames+=(sc.cdsc_absaddr.msf.second * CD_FRAMES);
    frames+=(sc.cdsc_absaddr.msf.minute * CD_FRAMES * 60);
  }
  else if(CDROM_RELATIVE==type){
    frames=sc.cdsc_reladdr.msf.frame;
    frames+=(sc.cdsc_reladdr.msf.second * CD_FRAMES);
    frames+=(sc.cdsc_reladdr.msf.minute * CD_FRAMES * 60);
  }
  else fatal_error("Unknown address type argument",0);
  return frames;
}

unsigned int cd_seconds_address(const int drive, const cdrom_addr_t type)
{
  unsigned long frames=cd_raw_address(drive,type);
  unsigned int seconds=(unsigned int)(frames/CD_FRAMES);
  if((frames%CD_FRAMES)>(CD_FRAMES/2))seconds++;
  return(seconds);
}
  
/*length in frames of CD up to the end of given track*/
static unsigned long cd_raw_track_length(const int drive, 
					 unsigned char track)
{
  unsigned long frames=0;
  struct cdrom_tocentry toc;
  toc.cdte_format=CDROM_MSF;
  /*We either want the beginning of the next track, or the leadout...*/
  toc.cdte_track=(track==
		  cd_end_track(drive,CDROM_LAST_TRACK))?CDROM_LEADOUT:track+1;

  if(ioctl(drive,CDROMREADTOCENTRY,&toc )){
    fatal_error("ioctl CDROMREADTOCENTRY",1);
  }
  /*There are CD_FRAMES frames a second, and 60 seconds a minute*/
  frames=toc.cdte_addr.msf.frame;
  frames+=(toc.cdte_addr.msf.second * CD_FRAMES);
  frames+=(toc.cdte_addr.msf.minute * CD_FRAMES * 60);
  return frames;
}

static unsigned long cd_frames_track_length(const int drive, 
					    unsigned char track)
{
  /*to work out length of given track in frames, we take the 
   *frames length of the end of track, and take away
   *the frames length of the end of track-1
   */
  unsigned long current=cd_raw_track_length(drive,track);
  unsigned long prev=cd_raw_track_length(drive,(track-1));
  if(prev>current)fatal_error("previous track is later than this one???",0);
  current-=prev;
  return current;
}

/*now track length to the nearest second*/
int cd_seconds_track_length(const int drive, unsigned char track)
{
  unsigned long frames=cd_frames_track_length(drive,track);
  /*There are CD_FRAMES a second*/
  int seconds=(int)(frames/CD_FRAMES);
  /*Now do some rounding*/
  if((frames%CD_FRAMES)>(CD_FRAMES/2))seconds++;
  return(seconds);
}

/*special case - just get the raw length of the last track, and 
 *work out how many seconds that is
 */
static int cd_seconds_disc_length(const int drive)
{ 
  unsigned long f=(cd_raw_track_length(drive,cd_end_track(drive,
							  CDROM_LAST_TRACK)));
  /*There are CD_FRAMES a second*/
  int seconds=(int)(f/CD_FRAMES);
  /*Now do some rounding*/
  if((f%CD_FRAMES)>(CD_FRAMES/2))seconds++;
  return(seconds);  
}

void cd_info_drive(const int drive)
{
  int s;
  unsigned char t,l;
  unsigned int rel,ab;
  int ts,ls;

  switch(s=cd_audio_status(drive)){
  case CDROM_AUDIO_PLAY:
  case CDROM_AUDIO_PAUSED:
    t=cd_current_track(drive);
    ts=cd_seconds_track_length(drive,t);
    l=cd_end_track(drive,CDROM_LAST_TRACK);
    ls=cd_seconds_disc_length(drive);
    rel=cd_seconds_address(drive, CDROM_RELATIVE);
    ab=cd_seconds_address(drive, CDROM_ABSOLUTE);
    fprintf(stdout,"%s track %d (of %d), %u:%.2u/%d:%.2d, (%u:%.2u/%d:%.2d)\n",
	    CDROM_AUDIO_PLAY==s?"Playing":"Paused on",
	    t,l,rel/60,rel%60,ts/60,ts%60,ab/60,ab%60,ls/60,ls%60);
    break;
  case CDROM_AUDIO_COMPLETED:
  case CDROM_AUDIO_NO_STATUS:
    l=cd_end_track(drive,CDROM_LAST_TRACK);
    ls=cd_seconds_disc_length(drive);
    fprintf(stdout,"Not playing. %d tracks, %d:%.2d\n",
	    l,ls/60,ls%60);
    break;
  case CDROM_AUDIO_INVALID:
    fprintf(stdout,"Audio not supported\n");
    break;
  case CDROM_AUDIO_ERROR:
    fprintf(stdout,"Audio playback halted due to error\n");
    break;
  default:
    fatal_error("Unknown drive status\n",0);
  }

  exit(0);
}

static cdrom_tracktype_t cd_type_track(const int drive, 
				       const unsigned char track)
{
  struct cdrom_tocentry toc;

  memset(&toc,0,sizeof(struct cdrom_tocentry));
  toc.cdte_format=CDROM_MSF;
  toc.cdte_track=(int)track;
  if(ioctl(drive,CDROMREADTOCENTRY,&toc )){
    fatal_error("ioctl CDROMREADTOCENTRY",1);
  }
  /*This bit (0x04) is defined in cdrom.h*/
  if(toc.cdte_ctrl & CDROM_DATA_TRACK)
    return CDROM_TRACKIS_DATA;
  else return CDROM_TRACKIS_AUDIO;
}

void cd_dir_drive(const int drive)
{
  int s,stopped=0;
  /*Last track/"active" track current being worked on/Track now playing*/
  unsigned char l=0,a=0,t=0;
  unsigned int rel=0;
  int ls,as;

  s=cd_audio_status(drive);
  l=cd_end_track(drive,CDROM_LAST_TRACK);
  ls=cd_seconds_disc_length(drive);

  switch(s){
  case CDROM_AUDIO_COMPLETED:
  case CDROM_AUDIO_NO_STATUS:
    stopped=1;
  case CDROM_AUDIO_PLAY:
  case CDROM_AUDIO_PAUSED:
    fprintf(stdout,"Track  Time (%d tracks / %d:%.2d)\n",
	    l,ls/60,ls%60);
    if(0==stopped){
      t=cd_current_track(drive);
      rel=cd_seconds_address(drive, CDROM_RELATIVE);
    }
    for(a=1;a<=l;a++){
      as=cd_seconds_track_length(drive,a);
      fprintf(stdout,"%.2u     %d:%.2d",a,as/60,as%60);
      /*This can only be true if either playing or paused*/
      if(a==t)
	fprintf(stdout," *%s %u:%.2u*",
		CDROM_AUDIO_PLAY==s?"Playing":"Paused at",
		rel/60,rel%60);
      else if(CDROM_TRACKIS_DATA==cd_type_track(drive,a))
	fprintf(stdout," [DATA]");
      fprintf(stdout,"\n");
    }
    break;
  case CDROM_AUDIO_INVALID:
    fprintf(stdout,"Audio not supported\n");
    break;
  case CDROM_AUDIO_ERROR:
    fprintf(stdout,"Audio playback halted due to error\n");
    break;
  default:
    fatal_error("Unknown drive status\n",0);
  }
}

void cd_capable_drive(const int drive, const char *path)
{
  int a=0,c=0,used=0;
  char *cols;
  struct winsize ws;

  /*unlike most other CDROM ioctls, this one returns the data as a 
   *return value *sigh*
   */
  if((a=(ioctl(drive,CDROM_GET_CAPABILITY)))==-1)
    fatal_error("ioctl CDROM_GET_CAPABILITY failed",1);
  
  if((cols=getenv("COLUMNS")))
    c=strtol(cols,NULL,10);
  else if((0==(ioctl(STDOUT_FILENO,TIOCGWINSZ,&ws))))
    c=ws.ws_col;
  /*if not set, guess at 80*/
  if(c==0)c=80;

  fprintf(stdout,"Capabilities of drive %s:\n",path);
  if(a & CDC_CLOSE_TRAY)used=display_capability("load, ",used,c);
  if(a & CDC_OPEN_TRAY)used=display_capability("eject, ",used,c);
  if(a & CDC_LOCK)used=display_capability("lock, ",used,c);
  if(a & CDC_SELECT_SPEED)used=display_capability("vary speed, ",used,c);
  if(a & CDC_SELECT_DISC)used=display_capability("select disc, ",used,c);
  if(a & CDC_MULTI_SESSION)used=display_capability("multi-session, ",used,c);
  if(a & CDC_MCN)used=display_capability("MCN, ",used,c);
  if(a & CDC_PLAY_AUDIO)used=display_capability("audio, ",used,c);
  if(a & CDC_RESET)used=display_capability("reset, ",used,c);
  if(a & CDC_IOCTLS)used=display_capability("non-standard ioctls, ",used,c);
  if(a & CDC_DRIVE_STATUS)used=display_capability("status, ",used,c);
  if(a & CDC_GENERIC_PACKET)used=display_capability("generic packets, ",used,c);
  if(a & CDC_CD_RW)used=display_capability("CDRW, ",used,c);
  if(a & CDC_CD_R)used=display_capability("CDR. ",used,c);
  if(a & CDC_DVD_R)used=display_capability("DVD-R write, ",used,c);
  if(a & CDC_DVD_RAM)used=display_capability("DVD-RAM write,",used,c);
  if(a & CDC_DVD_R)used=display_capability("DVD. ",used,c);
  printf("\n");
}

static int display_capability(const char *string, int used,const int cols)
{
  size_t l=strlen(string);
  if((used+l)>=cols){
    fprintf(stdout,"\n%s",string);
    /*I know this is safe, given the length of the strings being passed*/
    used=(int)l;
  }
  else{ 
    fprintf(stdout,"%s",string);
    used+=(int)l;
  }
  return(used);
}

void cd_multiple_tracks(const int drive,int **lt,int len,const int loop,
			const int rndm)
{
  int t;
  if(!(*lt)) *lt=list_all_tracks(drive,lt,&len);
  *lt=check_track_list(drive,lt,&len);
  /*Seed the RNG if needed*/
  if(rndm)srand((unsigned int)time(NULL));

  for(;;){
    /*list_next_track will exit if all tracks played 
     *and we're not looping
     */
    t=list_next_track(lt,&len,loop,rndm);
    wait_play_track(drive,t);
  }
}

static int *list_all_tracks(const int drive, int **lt, int *len)
{
  int a,i,n,z;
  a=(int)cd_end_track(drive,CDROM_FIRST_TRACK);
  z=(int)cd_end_track(drive,CDROM_LAST_TRACK);
  n=1+(z-a);
  if((*lt=(int *)malloc(n*(sizeof(int))))==NULL)
    fatal_error("Unable to allocate memory",1);
  for(i=0;i<n;i++)
    (*lt)[i]=i+a;
  *len=(n+1);
  return(*lt);
}

static int *trim_list(int **lt, int *len, int where)
{
  int i;

  for(i=where;i<((*len)-1);i++)
    (*lt)[i]=(*lt)[i+1];
  *len=(*len)-1;

  return(*lt);
}

/*remove any illegal track numbers or data tracks from the list*/
static int *check_track_list(const int drive, int **lt, int *len)
{
  int i,z;  

  z=(int)cd_end_track(drive,CDROM_LAST_TRACK);
  for(i=0;i<*len;i++){
    if(((*lt)[i])>z) *lt=trim_list(lt,len,i);
    else if(CDROM_TRACKIS_DATA==cd_type_track(drive,(*lt)[i]))
      *lt=trim_list(lt,len,i);
  }
  return(*lt);
}

static int *cycle_list(int **lt, int *len)
{
  int i;
  int t=*lt[0];
  for(i=0;i<((*len)-1);i++)
    (*lt)[i]=(*lt)[i+1];
  (*lt)[(*len)-1]=t;
  return(*lt);
}

static int list_next_track(int **lt,int *len,const int loop,const int rndm)
{
  int t,n;

  /*If we're finished, then quit*/
  if((*len)<1)exit(0);

  if(!rndm){
    t=(*lt)[0];
    if(!loop)*lt=trim_list(lt,len,0);
    else*lt=cycle_list(lt,len);
  }
  else{
    /*For commentary on this generation of random numbers, 
     *see rand(3) on linux, which refers to Numerical Recipies
     *in C by Press et al (2nd ed p277); basically, with older
     *rand() implementations, the higher-order bits are much more
     *random than the lower-order bits
     */
    n=(int)(((*len)-1)*(rand()/(RAND_MAX+1.0)));
    if(!loop){
      t=(*lt)[n];
      trim_list(lt,len,n);
    }
    else t=(*lt)[n];
  }
  return(t);
}

static void wait_play_track(const int drive, const int t)
{
  unsigned long l=cd_frames_track_length(drive,t);
  struct timeval tv;
  struct sigaction newh, oldh;
  fd_set fds;
  int a;

  tv.tv_sec=0;
  /*These are microseconds*/
  tv.tv_usec=(int)(1000000.0*l/CD_FRAMES);

  if(-1==(pipe(pipe_fds)))
    fatal_error("Unable to open pipe",1);

  /*install our signal handler for SIGINT and save the old one*/
  newh.sa_handler=sigint_handler;
  sigemptyset(&newh.sa_mask);
  newh.sa_flags=SA_RESTART; /*so select() doesn't get EINTR*/
  sigaction(SIGINT,&newh,&oldh);

  FD_ZERO(&fds);
  FD_SET(pipe_fds[0],&fds);

  cd_play_tracks(drive,t,t);
  /*Use select() to sleep. If we are sent SIGINT, then
   *we'll be woken up with data in pipe_fds[0]
   *in which case, we stop the drive and exit
   */
  a=select((pipe_fds[0])+1,&fds,NULL,NULL,&tv);

  if((a>0)||((a<0)&&(EINTR==errno))) cd_stop_drive(drive);
  else if(a<0) fatal_error("select() failed",1);
}

static void sigint_handler(int signum)
{
  if(-1==(write(pipe_fds[1],&signum,sizeof(int))))
    abort();
}
