/* $Id: network.c,v 1.4 2004/05/25 23:26:23 matthew Exp $
 *
 * mcdp - command line CD playing
 * Copyright (C) 2003 Matthew Vernon <matthew@gnu.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Log: network.c,v $
 * Revision 1.4  2004/05/25 23:26:23  matthew
 * hack to shut compiler up on ARM
 *
 * Revision 1.3  2003/11/23 14:57:18  matthew
 * lookup_freedb_host is a local function
 *
 * Revision 1.2  2003/11/23 13:38:09  matthew
 * lookup host and connect() to it
 *
 * Revision 1.1  2003/11/22 18:28:11  matthew
 * boilerplate
 *
 *
 */

#include "libmcdp.h"
#include "ferror.h"
#include "network.h"

#include <stdio.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

static struct in_addr *cd_net_lookup_freedb_host(const char *hostname);


struct in_addr *cd_net_lookup_freedb_host(const char *hostname)
{
  struct hostent *he;

  if(NULL==(he=gethostbyname(hostname))){
    switch(h_errno){
    case HOST_NOT_FOUND:
      fatal_error("Failed host lookup - host unknown",0);
      break;
    case NO_ADDRESS:
      fatal_error("Failed host lookup - no IP associated with hostname",0);
      break;
    case NO_RECOVERY:
      fatal_error("Failed host lookup - nameserver error",0);
      break;
    case TRY_AGAIN:
      fatal_error("Failed host lookup - \"Try again later\"",0);
      break;
    default:
      fatal_error("Failed host lookup - unknown error value",0);
    }
  }
  /*pick the first address returned*/
  return((struct in_addr *)(void *) he->h_addr_list[0]);
}

FILE *cd_net_connect(const char *hostname)
{
  struct in_addr *ia;
  struct sockaddr_in si;
  int s;
  FILE *f;

  ia=cd_net_lookup_freedb_host(hostname);

  if(-1==(s=socket(PF_INET, SOCK_STREAM, 0)))
    fatal_error("Opening socket",1);

  si.sin_family=AF_INET;
  si.sin_port=htons(FREEDB_PORT);
  si.sin_addr=*ia;

  if(-1==(connect(s,(struct sockaddr *) &si,sizeof(si))))
    fatal_error("Connecting to server",1);

  if(NULL==(f=(fdopen(s,"w+"))))
    fatal_error("Opening FILE * on socket",1);

  return(f);
}
