/* $Id: ui.c,v 1.23 2005/05/15 15:40:59 matthew Exp $
 *
 * mcdp - command line CD playing
 * Copyright (C) 2003 Matthew Vernon <matthew@gnu.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Log: ui.c,v $
 * Revision 1.23  2005/05/15 15:40:59  matthew
 * correct an off-by-one error.
 *
 * Revision 1.22  2003/11/23 16:33:54  matthew
 * rudimentary CDDB support
 *
 * Revision 1.21  2003/11/22 18:27:32  matthew
 * Add code to generate the CDDB query string
 *
 * Revision 1.20  2003/11/22 16:40:13  matthew
 * Support playing tracks if a numerical argument is supplied.
 *
 * Revision 1.19  2003/09/01 23:40:52  matthew
 * Add support for generating freedb &c. discids
 *
 * Revision 1.18  2003/06/03 14:13:01  matthew
 * Remember how to write C
 *
 * Revision 1.17  2003/06/03 14:12:25  matthew
 * not apt, dammit
 *
 * Revision 1.16  2003/04/20 17:12:08  matthew
 * Add new ui feature
 *
 * Revision 1.15  2003/04/19 19:07:58  matthew
 * looping and random play functions. Debugging still enabled
 *
 * Revision 1.14  2003/04/18 17:25:08  matthew
 * Added capabilities query
 *
 * Revision 1.13  2003/04/06 16:50:43  matthew
 * Added previous and next track code
 *
 * Revision 1.12  2003/04/05 09:33:09  matthew
 * directory function written, needs debugging
 *
 * Revision 1.11  2003/04/05 08:53:07  matthew
 * Reduce number of strlen calls
 *
 * Revision 1.10  2003/04/05 01:40:11  matthew
 * Added status query functionality
 *
 * Revision 1.9  2003/04/04 19:34:28  matthew
 * correct track specification checking logic
 *
 * Revision 1.8  2003/04/04 19:26:04  matthew
 * initialise track number variables
 *
 * Revision 1.7  2003/04/04 16:39:03  matthew
 * Added track specification parsing
 *
 * Revision 1.6  2003/04/04 16:14:37  matthew
 * work in progress on track parsing
 *
 * Revision 1.5  2003/04/04 15:34:40  matthew
 * Added parsing of non-option arguments
 *
 * Revision 1.4  2003/04/04 15:20:52  matthew
 * no non-implemented action values
 *
 * Revision 1.3  2003/04/04 15:19:16  matthew
 * Added pause/resume functionality
 *
 * Revision 1.2  2003/04/04 14:44:06  matthew
 * Add copyright notice and CVS variables
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <rgetopt.h>
#include <ctype.h>

#include "ui.h"
#include "libmcdp.h"
#include "version.h"
#include "ferror.h"

static void guess_action(const char *string);
static void parse_tracks_spec(const char *string,int *from,int *to);
static void parse_list(const char **strings,int **numbers);
static void sheep(void);
static void cow(void);

int main(int argc, const char *const *argv) {
  int i,j=0,drive_fd,n,a=0,z=0;

  for(i=0;i<MCD_ACTIONS;i++){
    action[i]=0;
  }

  switch(n=rgetopt(argc,argv,options)) {
  case rgetopt_ret_error:
    fprintf(stderr, "Syntax error in arguments\n"
	    "Try %s --help for more information\n", argv[0]);
    exit(1);
  case rgetopt_ret_version:
    fprintf(stdout,"mcdp %s\n", VERSION);
    exit(0);
  case rgetopt_ret_help:
    fprintf(stdout,"Usage:\n"
	    "mcdp [options ...] [arguments ...]\n"
	    "mcdp --help\n"
	    "mcdp --version\n\n");
    rgetopt_printhelp(stdout, options);
    exit(0);
  }
  /*Look for a non-option argument*/
  if((n!=argc)&&(argc-n>1))
    fatal_error("too many non-option arguments",0);
  if((n!=argc)&&(argc-n==1))
    guess_action(argv[n]);

  if(list_tracks_string)parse_list(list_tracks_string,&list_tracks);

  /*check we've only been asked to do one thing*/
  for(i=0;i<MCD_ACTIONS;i++){
    if(action[i]==1)j++;
  }
  if((j>1)&&(!action[MCD_LOOP])) fatal_error("More than one action specified!",0);
  /*default action is to play the CD*/
  if(j==0)action[MCD_PLAY]=1;

  /*open the CD drive*/
  if(device_path)drive_fd=cd_open_device(device_path);
  else drive_fd=cd_open_device(DEFAULT_DEVICE);

  /*Do something with it :-)*/
  if(action[MCD_LOAD])cd_load_drive(drive_fd);
  else if(action[MCD_EJECT])cd_eject_drive(drive_fd);
  else if(action[MCD_STOP])cd_stop_drive(drive_fd);
  else if((action[MCD_LOOP])||(action[MCD_RANDOM])||(list_tracks))
    cd_multiple_tracks(drive_fd,&list_tracks,list_length,action[MCD_LOOP],
		       action[MCD_RANDOM]);
  else if(action[MCD_PLAY]){
    if(tracks_arg)parse_tracks_spec(tracks_arg,&a,&z);
    cd_play_tracks(drive_fd,a,z);
  }
  else if(action[MCD_DIR]) cd_dir_drive(drive_fd);
  else if(action[MCD_PAUSE])cd_pause_drive(drive_fd);
  else if(action[MCD_INFO])cd_info_drive(drive_fd);
  else if(action[MCD_NEXT])cd_next_track(drive_fd);
  else if(action[MCD_PREV])cd_prev_track(drive_fd);
  else if(action[MCD_CAPABLE]){
    if(device_path)cd_capable_drive(drive_fd,device_path);
    else cd_capable_drive(drive_fd,DEFAULT_DEVICE);
  }
  else if(action[MCD_DISCID])cd_discid(drive_fd);
  else if(action[MCD_QUERY_STRING])cd_output_query(drive_fd);
  else if(action[MCD_LOOKUP])cd_lookup_disc(drive_fd);
  else fatal_error("No applicable action found",0);

  exit(0);
}

static void guess_action(const char *string)
{
  size_t l;

  l=strlen(string);

  if(0==(strncasecmp(string,"play",l)))
    action[MCD_PLAY]=1;
  else if(0==(strncasecmp(string,"pause",l)))
    action[MCD_PAUSE]=1;
  else if(0==(strncasecmp(string,"stop",l)))
    action[MCD_STOP]=1;
  else if(0==(strncasecmp(string,"load",l)))
    action[MCD_LOAD]=1;
  else if(0==(strncasecmp(string,"eject",l)))
    action[MCD_EJECT]=1;
  else if(0==(strncasecmp(string,"info",l)))
    action[MCD_INFO]=1;
  else if(0==(strncasecmp(string,"dir",l)))
    action[MCD_DIR]=1;
  else if(0==(strncasecmp(string,"next",l)))
    action[MCD_NEXT]=1;
  else if(0==(strncasecmp(string,"prev",l)))
    action[MCD_PREV]=1;  
  else if(0==(strncasecmp(string,"capable",l)))
    action[MCD_CAPABLE]=1;
  else if(0==(strncasecmp(string,"random",l)))
    action[MCD_RANDOM]=1;
  else if(0==(strncasecmp(string,"loop",l)))
    action[MCD_LOOP]=1;
  else if(0==(strncasecmp(string,"discid",l)))
    action[MCD_DISCID]=1;
  else if(0==(strncasecmp(string,"query-string",l)))
    action[MCD_QUERY_STRING]=1;
  else if(0==(strncasecmp(string,"lookup",l)))
    action[MCD_LOOKUP]=1;
  else if(isdigit(string[0]))
    tracks_arg=strdup(string);
  else if(0==(strncasecmp(string,"baa",l)))
    sheep();
  else if(0==(strncasecmp(string,"moo",l)))
    cow();
  else fatal_error("Unrecognised non-option argument",0);
}

static void parse_tracks_spec(const char *string,int *from,int *to)
{
  char *endptr;
  long int val;

  val=strtol(string,&endptr,10);
  if(((LONG_MIN==val)||(LONG_MAX==val))&&(ERANGE==errno))
    fatal_error("Failure to convert track specification",1);
  if(val<0)
    fatal_error("Track numbers must be positive",0);
  *from=(int)val;
  if(*endptr){
    endptr++;
    if(*endptr){
      val=strtol(endptr,NULL,10);
      if(((LONG_MIN==val)||(LONG_MAX==val))&&(ERANGE==errno))
	fatal_error("Failure to convert track specification",1);
      if(val<0)
	fatal_error("Track numbers must be positive",0);
      *to=(int)val;
    }
    else *to=0;
  }
  /*If just 1 No. is specified, then play just that track*/
  else *to=*from;
  
  if((*to)&&(*from>*to))fatal_error("Second track number must not be lower than first",0);
}

/*Take the list of tracks as a string we got from rgetopt, 
 *and turn it into an array of numbers
 */
static void parse_list(const char **strings,int **numbers)
{
  int i;
  long int l;

  for(i=0;strings[i];i++); /*Make i equal to the number of entries in strings*/
  if((*numbers=(int *)malloc(i*sizeof(int)))==NULL)
    fatal_error("Malloc failed",1);
  for(i=0;strings[i];i++){
    l=strtol(strings[i],NULL,10);
    if(((LONG_MIN==l)||(LONG_MAX==l))&&(ERANGE==errno))
      fatal_error("Failure to parse specified track list",1);
    if(l<0)
      fatal_error("Track numbers must be positive",0);
    (*numbers)[i]=(int)l;
  }
  list_length=i;
};

static void sheep(void)
{
  printf("                 /\\___\n      @@@@@@@@@@@  O  \\   \n");
  printf("   @@@@@@@@@@@@@@@____/\n  @@@@@@@@@@@@@@@@\n");
  printf("  @ @@@@@@@@@@@@\n     ||      ||\n     ~~      ~~\n");
  printf("There is an ASCII sheep in every flock. Baa!\n");
  exit(0);
}   

static void cow(void)
{
  printf("This is a CD player, not a package management tool!\n");
  exit(0);
}
   
   
   
   

