/* $Id: ui.h,v 1.11 2003/11/23 16:33:54 matthew Exp $
 *
 * mcdp - command line CD playing
 * Copyright (C) 2003 Matthew Vernon <matthew@gnu.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __UI_H__
#define __UI_H__

#define DEFAULT_DEVICE "/dev/cdrom"

typedef enum{
  MCD_PLAY,
    MCD_PAUSE,
    MCD_STOP,
    MCD_LOAD,
    MCD_EJECT,
    MCD_INFO,
    MCD_DIR,
    MCD_NEXT,
    MCD_PREV,
    MCD_CAPABLE,
    MCD_RANDOM,
    MCD_LOOP,
    MCD_DISCID,
    MCD_QUERY_STRING,
    MCD_LOOKUP,
    MCD_ACTIONS,
    } mcd_action_t;

    int action[MCD_ACTIONS];

static const char *device_path=NULL;
static const char *tracks_arg=NULL;
static const char **list_tracks_string=NULL;
static int *list_tracks=NULL;
static int list_length=0;

static const struct rgetopt options[] = {
  {"--device",       "path to cdrom device",       'd', RGETOPT_FILE(&device_path,rgetopt_writable)},
  {"--tracks",       "play specified track(s)",    't', RGETOPT_REGEXP(&tracks_arg, "(^[0-9]*$)|(^[0-9]*,[0-9]*$)")},
  {"--play",         "play CD",                    'p', RGETOPT_INT(&(action[MCD_PLAY]),1)}, 
  {"--pause",        "pause/resume CD",            0,   RGETOPT_INT(&(action[MCD_PAUSE]),1)},
  {"--stop",         "stop CD",                    's', RGETOPT_INT(&(action[MCD_STOP]),1)},
  {"--load",         "load CD",                    'l', RGETOPT_INT(&(action[MCD_LOAD]),1)},
  {"--eject",        "eject CD",                   'e', RGETOPT_INT(&(action[MCD_EJECT]),1)},
  {"--info",         "show CD status",             'i', RGETOPT_INT(&(action[MCD_INFO]),1)},
  {"--dir",          "show full CD contents",      'd', RGETOPT_INT(&(action[MCD_DIR]),1)},
  {"--next",         "skip forward one track",     'n', RGETOPT_INT(&(action[MCD_NEXT]),1)},
  {"--prev",         "skip back one track",        0,   RGETOPT_INT(&(action[MCD_PREV]),1)},
  {"--capable",      "Display drive capabilities", 'c', RGETOPT_INT(&(action[MCD_CAPABLE]),1)} ,
  {"--list",         "Specify a list of tracks",   0,   RGETOPT_LIST(&list_tracks_string)},
  {"--random",       "Play tracks in random order",'r', RGETOPT_INT(&(action[MCD_RANDOM]),1)},
  {"--loop",         "Loop over tracks for ever",  0,   RGETOPT_INT(&(action[MCD_LOOP]),1)},
  {"--discid",       "Generate Disc-ID (for freedb, etc.)", 0, RGETOPT_INT(&(action[MCD_DISCID]),1)},
  {"--query-string", "Generate freedb query string", 0, RGETOPT_INT(&(action[MCD_QUERY_STRING]),1)},
  {"--lookup",       "Lookup disc on freedb",      0,   RGETOPT_INT(&(action[MCD_LOOKUP]),1)},
  {"",              NULL,                       0,   RGETOPT_HELP},
  {"",              NULL,                       0,   RGETOPT_VERSION},
  {RGETOPT_END},
};


#endif /*__UI_H__*/
